from setuptools import setup


setup(
    name='pyretry',
    version="1.0.0",
    description='Separate your retry logic from your business logic',
    author='Bob Renwick',
    author_email='bob.renwick@gmail.com',
    url='https://github.com/bobbyrenwick/pyretry',
    packages=['pyretry'],
    classifiers=(
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
    ),
)
